'''title'''
__title__ = 'pikachuwechat'
'''description'''
__description__ = 'PikachuWeChat: Play WeChat with itchat-uos.'
'''url'''
__url__ = 'https://github.com/CharlesPikachu/pikachuwechat'
'''version'''
__version__ = '0.1.1'
'''author'''
__author__ = 'Zhenchao Jin'
'''email'''
__email__ = 'charlesblwx@gmail.com'
'''license'''
__license__ = 'MIT License'
'''copyright'''
__copyright__ = 'Copyright 2021 Zhenchao Jin'


'''
Function:
    微信小助手
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import warnings
if __name__ == '__main__':
    from modules import *
    from __init__ import __version__
else:
    from .modules import *
    from .__init__ import __version__
warnings.filterwarnings('ignore')


'''basic info'''
BASICINFO = '''************************************************************
Function: 微信小助手 V%s
Author: Charles
微信公众号: Charles的皮卡丘
************************************************************''' % (__version__)


'''微信小助手'''
class pikachuwechat():
    def __init__(self, **kwargs):
        for key, value in kwargs.items(): setattr(self, key, value)
        self.supported_funcs = self.initializeallfuncs()
    '''开发者调用对应的功能'''
    def execute(self, func_type=None, config={}):
        assert func_type in self.supported_funcs, 'unsupport func_type %s...' % func_type
        client = self.supported_funcs[func_type](**config)
        client.run()
    '''初始化所有功能'''
    def initializeallfuncs(self):
        supported_funcs = {
            'AutoReply': AutoReply,
            'AntiWithdrawal': AntiWithdrawal,
            'AnalysisFriends': AnalysisFriends,
        }
        return supported_funcs
    '''获得所有支持的功能信息'''
    def getallsupported(self):
        all_supports = {}
        for key, value in self.supported_funcs.items():
            all_supports[value.func_name] = key
        return all_supports
    '''repr'''
    def __repr__(self):
        return BASICINFO


'''run'''
if __name__ == '__main__':
    import random
    wechat_helper = pikachuwechat()
    all_supports = wechat_helper.getallsupported()
    wechat_helper.execute(random.choice(list(all_supports.values())))


'''
Function:
    微信好友分析
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import os
import itchat
from ..utils import Logger, checkDir
from pyecharts.charts import Pie, Map
from pyecharts import options as opts


'''微信好友分析'''
class AnalysisFriends():
    func_name = '微信好友分析'
    logger_handle = Logger(func_name+'.log')
    def __init__(self, **kwargs):
        self.options = kwargs
        self.savedir = kwargs.get('savedir', 'results')
        checkDir(self.savedir)
    '''外部调用运行'''
    def run(self):
        # 登录
        try: itchat.auto_login(hotReload=True)
        except: itchat.auto_login(hotReload=True, enableCmdQR=True)
        # 获得所有好友信息
        AnalysisFriends.logger_handle.info('run getFriendsInfo...')
        friends_info = self.getFriendsInfo()
        # 分析好友地域分布
        AnalysisFriends.logger_handle.info('run analysisArea...')
        self.analysisArea(friends_info=friends_info)
        # 分析好友性别分布
        AnalysisFriends.logger_handle.info('run analysisSex...')
        self.analysisSex(friends_info=friends_info)
    '''分析好友地域分布'''
    def analysisArea(self, title='分析好友地域分布', friends_info=None):
        area_infos = {'未知': 0}
        for item in friends_info.get('province'):
            if not item: area_infos['未知'] += 1
            else:
                if item in area_infos: area_infos[item] += 1
                else: area_infos[item] = 1
        map_ = Map(init_opts=dict(theme='purple-passion', page_title=title))
        map_.add(title, data_pair=tuple(zip(area_infos.keys(), area_infos.values())), maptype='china')
        map_.set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            visualmap_opts=opts.VisualMapOpts(max_=200),
        )
        map_.render(os.path.join(self.savedir, '%s.html' % title))
    '''分析好友性别分布'''
    def analysisSex(self, title='分析好友性别分布', friends_info=None):
        sex_infos = {'男': 0, '女': 0, '未知': 0}
        for item in friends_info.get('sex'):
            if item == 0: sex_infos['未知'] += 1
            elif item == 1: sex_infos['男'] += 1
            elif item == 2: sex_infos['女'] += 1
        pie = Pie(init_opts=dict(theme='westeros', page_title=title)).add(title, data_pair=tuple(zip(sex_infos.keys(), sex_infos.values())), rosetype='area')
        pie.set_global_opts(title_opts=opts.TitleOpts(title=title))
        pie.render(os.path.join(self.savedir, '%s.html' % title))
    '''获得所需的微信好友信息'''
    def getFriendsInfo(self):
        friends = itchat.get_friends()
        friends_info = dict(
            province=self.getKeyInfo(friends, "Province"),
            city=self.getKeyInfo(friends, "City"),
            nickname=self.getKeyInfo(friends, "Nickname"),
            sex=self.getKeyInfo(friends, "Sex"),
            signature=self.getKeyInfo(friends, "Signature"),
            remarkname=self.getKeyInfo(friends, "RemarkName"),
            pyquanpin=self.getKeyInfo(friends, "PYQuanPin")
        )
        return friends_info
    '''根据key值得到对应的信息'''
    def getKeyInfo(self, friends, key):
        return list(map(lambda friend: friend.get(key), friends))


'''
Function:
    微信消息防撤回
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import re
import os
import time
import itchat
from ..utils import Logger
from itchat.content import TEXT, PICTURE, FRIENDS, CARD, MAP, SHARING, RECORDING, ATTACHMENT, VIDEO, NOTE


'''微信消息防撤回'''
class AntiWithdrawal():
    msg_infos = {}
    face_package = None
    func_name = '微信消息防撤回'
    logger_handle = Logger(func_name+'.log')
    def __init__(self, **kwargs):
        self.options = kwargs
    '''外部调用运行'''
    def run(self):
        try: itchat.auto_login(hotReload=True)
        except: itchat.auto_login(hotReload=True, enableCmdQR=True)
        '''保存接收到的信息'''
        @itchat.msg_register([TEXT, PICTURE, FRIENDS, CARD, MAP, SHARING, RECORDING, ATTACHMENT, VIDEO], isFriendChat=True, isGroupChat=True, isMpChat=True)
        def savemsg(msg):
            msg_receive_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            # 获得消息发送者信息
            if 'ActualNickName' in msg:
                msg_from_nickname = msg['ActualNickName']
                msg_from_username = msg['ActualUserName']
                friends = itchat.get_friends(update=True)
                msg_from = msg_from_nickname
                for friend in friends:
                    if msg_from_username == friend['UserName']:
                        if friend['RemarkName']:
                            msg_from = friend['RemarkName']
                        else:
                            msg_from = friend['NickName']
                        break
                groups = itchat.get_chatrooms(update=True)
                group_name = ''
                for group in groups:
                    if msg['FromUserName'] == group['UserName']:
                        group_name = group['NickName']
                        group_menber_count = group['MemberCount']
                        break
                if not group_name: group_name = '未命名群聊'
                group_name = group_name + '(%s人)' % str(group_menber_count)
                msg_from = group_name + '------->' + msg_from
            else:
                try:
                    msg_from = itchat.search_friends(userName=msg['FromUserName'])['RemarkName']
                    if not msg_from: msg_from = itchat.search_friends(userName=msg['FromUserName'])['NickName']
                except:
                    msg_from = 'WeChat Official Accounts'
            # 获得消息时间
            msg_send_time, msg_id, msg_content, msg_link = msg['CreateTime'], msg['MsgId'], None, None
            # 文本或者好友推荐
            if msg['Type'] in ['Text', 'Friends']:
                msg_content = msg['Text']
                AntiWithdrawal.logger_handle.info(f'Msg Type: Text or Friends, \n\tMsg Content: {msg_content}')
            # 附件/视频/图片/语音
            elif msg['Type'] in ['Attachment', 'Video', 'Picture', 'Recording']:
                msg_content = msg['FileName']
                msg['Text'](str(msg_content))
                AntiWithdrawal.logger_handle.info(f'Msg Type: Attachment or Video or Picture or Recording, \n\tMsg Content: {msg_content}')
            # 位置信息
            elif msg['Type'] in ['Map']:
                x, y, location = re.search("<location x=\"(.*?)\" y=\"(.*?)\".*label=\"(.*?)\".*", msg['OriContent']).group(1, 2, 3)
                if location is None: msg_content = '纬度:' + x.__str__() + ', 经度:' + y.__str__()
                else: msg_content = location
                AntiWithdrawal.logger_handle.info(f'Msg Type: Map, \n\tMsg Content: {msg_content}')
            # 分享的音乐/文章
            elif msg['Type'] in ['Sharing']:
                msg_content, msg_link = msg['Text'], msg['Url']
                AntiWithdrawal.logger_handle.info(f'Msg Type: Sharing, \n\tMsg Content: {msg_content}')
            AntiWithdrawal.face_package = msg_content
            AntiWithdrawal.msg_infos.update({
                msg_id: {
                    'msg_from': msg_from,
                    'msg_send_time': msg_send_time,
                    'msg_receive_time': msg_receive_time,
                    'msg_type': msg['Type'],
                    'msg_content': msg_content,
                    'msg_link': msg_link,
                }
            })
            AntiWithdrawal.update()
        '''监听是否有消息撤回'''
        @itchat.msg_register(NOTE, isFriendChat=True, isGroupChat=True, isMpChat=True)
        def monitor(msg):
            if '撤回了一条消息' in msg['Content']:
                withdraw_msg_id = re.search("\<msgid\>(.*?)\<\/msgid\>", msg['Content']).group(1)
                withdraw_msg = AntiWithdrawal.msg_infos.get(withdraw_msg_id)
                AntiWithdrawal.logger_handle.info(f'{withdraw_msg} has been withdrawn')
                if len(withdraw_msg_id) < 11:
                    itchat.send_file(AntiWithdrawal.face_package, toUserName='filehelper')
                else:
                    prompt_list = [
                        withdraw_msg.get('msg_from') + '撤回了一条消息',
                        '消息类型: ' + withdraw_msg.get('msg_type'),
                        '接收时间: ' + withdraw_msg.get('msg_receive_time'),
                        '消息内容: ' + withdraw_msg.get('msg_content'),
                    ]
                    prompt = '\n'.join(prompt_list)
                    if withdraw_msg['msg_type'] in ['Sharing']: prompt += '\n链接: ' + withdraw_msg.get('msg_link')
                    itchat.send_msg(prompt, toUserName='filehelper')
                    if withdraw_msg['msg_type'] in ['Attachment', 'Video', 'Picture', 'Recording']:
                        file_ = '@fil@%s' % (withdraw_msg['msg_content'])
                        itchat.send(msg=file_, toUserName='filehelper')
                        os.remove(withdraw_msg['msg_content'])
                    AntiWithdrawal.msg_infos.pop(withdraw_msg_id)
        itchat.run()
    '''更新消息(删除5分钟之前的消息)'''
    @staticmethod
    def update():
        need_del_msgs = []
        for msg in AntiWithdrawal.msg_infos:
            msg_time_interval = int(time.time()) - AntiWithdrawal.msg_infos[msg]['msg_send_time']
            if msg_time_interval > 5 * 60: need_del_msgs.append(msg)
        for msg in need_del_msgs: AntiWithdrawal.msg_infos.pop(msg)


'''
Function:
    微信消息自动回复
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import itchat
from ..utils import Logger
from itchat.content import TEXT, PICTURE, FRIENDS, CARD, MAP, SHARING, RECORDING, ATTACHMENT, VIDEO, NOTE


'''自动回复的规则'''
AUTO_REPLY_RULES = {'工作': '[自动回复]: 您好, 我现在有事不在, 请稍后联系我.'}


'''微信消息自动回复'''
class AutoReply():
    func_name = '微信消息自动回复'
    logger_handle = Logger(func_name+'.log')
    def __init__(self, **kwargs):
        self.options = kwargs
        if 'auto_reply_rules' in kwargs:
            global AUTO_REPLY_RULES
            AUTO_REPLY_RULES = kwargs['auto_reply_rules']
    '''外部调用运行'''
    def run(self):
        try: itchat.auto_login(hotReload=True)
        except: itchat.auto_login(hotReload=True, enableCmdQR=True)
        '''消息自动回复'''
        @itchat.msg_register([TEXT, PICTURE, FRIENDS, CARD, MAP, SHARING, RECORDING, ATTACHMENT, VIDEO], isFriendChat=True)
        def autoreplay(msg):
            global AUTO_REPLY_RULES
            if msg['Text'] in AUTO_REPLY_RULES:
                itchat.send(AUTO_REPLY_RULES[msg['Text']], msg['FromUserName'])
                AutoReply.logger_handle.info('Msg: %s,\nReplay: %s' % (msg['Text'], AUTO_REPLY_RULES[msg['Text']]))
        itchat.run()